using System;
using System.ComponentModel;
using System.Collections.Generic;
using System.Diagnostics;
using System.Text;
using System.Windows.Forms;
using System.Drawing;
using System.Deployment.Application;
using System.Reflection;
using DigitaAdvantage.ClickOnce.Forms;
using System.Threading;
using System.Net;
using System.Xml;
using System.IO;

namespace DigitaAdvantage.ClickOnce
{
    /// <summary>
    /// vOɂ錻݂̔zu̍XVƁAKvɉt@C̃_E[hT|[g܂B̃NX͌pł܂B
    /// </summary>
    public sealed class ClickOnceAppDeployment
    {
        private static readonly object lockObject;
        private static ClickOnceAppDeployment currentClickOnceAppDeployment;

        private ApplicationDeployment appDeployment;
        private bool appDeployed;

        private ClickOnceDownloadDialog clickOnceDialog;
        private List<Form> changedForms;
        private bool showingClickOnceDialog;
        private string currentLogFilePath;
        private string currentPublisherName;
        private string currentProductName;
        private string currentSupportUrl;
        private string currentServerName;
        private string currentIconFilePath;
        private bool currentInstallMode;
        private ProcessingMode currentProcessingMode;
        private LocalUpdateCheckInfo currentUpdateCheckInfo;

        #region qNX
        private class LocalUpdateCheckInfo
        {
            private bool currentIsMemberFilled;
            private Version currentAvailableVersion;
            private bool currentIsUpdateRequired;
            private Version currentMinimumRequiredVersion;
            private bool currentUpdateAvailable;
            private long currentUpdateSize;

            public LocalUpdateCheckInfo()
            {
                InitializeData();
            }

            public void InitializeData()
            {
                currentIsMemberFilled = false;
                currentAvailableVersion = null;
                currentIsUpdateRequired = false;
                currentMinimumRequiredVersion = null;
                currentUpdateAvailable = false;
                currentUpdateSize = 0;
            }

            public bool IsMemberFilled
            {
                get { return currentIsMemberFilled; }
            }

            public Version AvailableVersion
            {
                get { return currentAvailableVersion; }
                set { currentAvailableVersion = value; }
            }

            public bool IsUpdateRequired
            {
                get { return currentIsUpdateRequired; }
                set { currentIsUpdateRequired = value; }
            }

            public Version MinimumRequiredVersion
            {
                get { return currentMinimumRequiredVersion; }
                set { currentMinimumRequiredVersion = value; }
            }

            public bool UpdateAvailable
            {
                get { return currentUpdateAvailable; }
                set { currentIsMemberFilled = true; currentUpdateAvailable = value; }
            }

            public long UpdateSizeBytes
            {
                get { return currentUpdateSize; }
                set { currentUpdateSize = value; } 
            }
        }
        #endregion

        #region 
        static ClickOnceAppDeployment()
        {
            lockObject = new object();
            currentClickOnceAppDeployment = null;
        }

        private ClickOnceAppDeployment()
        {
            appDeployed = ApplicationDeployment.IsNetworkDeployed;
            if (appDeployed)
            {
                appDeployment = ApplicationDeployment.CurrentDeployment;

                appDeployment.CheckForUpdateProgressChanged += new DeploymentProgressChangedEventHandler(appDeployment_CheckForUpdateProgressChanged);
                appDeployment.CheckForUpdateCompleted += new CheckForUpdateCompletedEventHandler(appDeployment_CheckForUpdateCompleted);

                appDeployment.DownloadFileGroupProgressChanged += new DeploymentProgressChangedEventHandler(appDeployment_DownloadFileGroupProgressChanged);
                appDeployment.DownloadFileGroupCompleted += new DownloadFileGroupCompletedEventHandler(appDeployment_DownloadFileGroupCompleted);

                appDeployment.UpdateProgressChanged += new DeploymentProgressChangedEventHandler(appDeployment_UpdateProgressChanged);
                appDeployment.UpdateCompleted += new AsyncCompletedEventHandler(appDeployment_UpdateCompleted);
            }

            clickOnceDialog = null;
            changedForms = new List<Form>();
            showingClickOnceDialog = false;
            currentLogFilePath = "";
            currentPublisherName = "";
            currentProductName = "";
            currentSupportUrl = "";
            currentServerName = "";
            currentIconFilePath = "";
            currentInstallMode = false;

            ProcessingMode = ProcessingMode.NoAction;
            currentUpdateCheckInfo = new LocalUpdateCheckInfo();
        }
        #endregion

        #region IsFileGroupDownloaded\bh̏
        /// <summary>
        /// NCAg Rs[^ɁAOtt@C O[vɃ_E[hĂ邩ǂ`FbN܂B
        /// </summary>
        /// <param name="groupName">
        /// _E[ht@C̖OtO[vB
        /// ClickOnce AvP[V "optional" Ƃă}[NꂽׂẴt@Cɂ́AO[vKvłB
        /// </param>
        /// <returns>
        /// ̃AvP[V݂̌̃o[Wɑ΂āA
        /// t@C O[vɃ_E[hĂꍇ trueBȊȌꍇ falseB
        /// AvP[V̐Vo[WCXg[ĂA
        /// ̐Vo[WŃt@C O[ṽt@Cɑ΂āA
        /// ǉA폜A܂͕ύXsĂȂꍇ true Ԃ܂B
        /// </returns>
        /// <exception cref="System.Deployment.Application.InvalidDeploymentException">
        /// groupName ́AAvP[V }jtFXgɒ`Ăt@C O[vł͂܂B
        /// </exception>
        public bool IsFileGroupDownloaded(string groupName)
        {
            return appDeployment.IsFileGroupDownloaded(groupName);
        }
        #endregion

        #region DownloadFileGroup\bh̏
        /// <summary>
        /// ÃIvV t@CKvɉă_E[h܂B
        /// </summary>
        /// <param name="groupName">
        /// _E[ht@C̖OtO[vB
        /// ClickOnce AvP[V "optional" Ƃă}[NꂽׂẴt@Cɂ́AO[vKvłB
        /// </param>
        /// <exception cref="System.ArgumentNullException">
        /// groupName p[^ null ܂͒ 0 łB
        /// </exception>
        public bool DownloadFileGroup(string groupName)
        {
            DeploymentMode mode = DeploymentMode.Download;
            this.ProcessingMode = ProcessingMode.NoAction;

            bool isDialogOpener = EnterClickOnceDialog();

            try
            {
                // GetDepolymentManifestInfo\bh͓Ǝ̂߃O𐶐ȂȂB
                // ]ł̓G[͍sȂBG[ׂ͂Ĉȍ~ClickOnce APIŔB
                GetDepolymentManifestInfo();
                clickOnceDialog.PublisherName = currentPublisherName;
                clickOnceDialog.AppProductName = currentProductName;
                clickOnceDialog.SupportUrl = currentSupportUrl;
                clickOnceDialog.ServerName = currentServerName;
                clickOnceDialog.IconFilePath = currentIconFilePath;
                if (currentInstallMode)
                {
                    mode = DeploymentMode.Install;
                }
                clickOnceDialog.HeaderTitle = DeploymentUtil.FormatHeaderTitleText(mode, currentProductName);
                clickOnceDialog.HeaderMessage = DeploymentUtil.FormatHeaderMessageText(mode);                
                clickOnceDialog.DeploymentMode = mode;

                using (ClickOnceLog log = new ClickOnceLog())
                {
                    currentLogFilePath = log.LogFilePath;

                    currentUpdateCheckInfo.InitializeData();

                    ProcessingMode = ProcessingMode.Processing;
                    clickOnceDialog.IsCanceled = false;

                    ChanteToUpdatingPage();

                    appDeployment.DownloadFileGroupAsync(groupName);

                    if (WaitAsyncMethod() == false)
                    {
                        return false;
                    }
                }
            }
            finally
            {
                CompleteProgressMessage();

                ExitClickOnceDialog(isDialogOpener);
            }

            if (ProcessingMode == ProcessingMode.Completed)
            {
                return true;
            }

            return false;
        }

        void appDeployment_DownloadFileGroupProgressChanged(object sender, DeploymentProgressChangedEventArgs e)
        {
            lock (lockObject)
            {
                if (clickOnceDialog.IsCanceled)
                {
                    appDeployment.DownloadFileGroupAsyncCancel(e.Group);
                    clickOnceDialog.IsCanceled = false;
                }

                UpdateProgressMessage(e.State, e.ProgressPercentage, e.BytesCompleted, e.BytesTotal, clickOnceDialog.DeploymentMode);
            }
        }

        void appDeployment_DownloadFileGroupCompleted(object sender, DownloadFileGroupCompletedEventArgs e)
        {
            lock (lockObject)
            {
                if (e.Error != null)
                {
                    ChangeToErrorPage("t@C O[ṽ_E[hɎs܂B", e.Error.Message);
                    return;
                }
                else if (e.Cancelled == true)
                {
                    this.ProcessingMode = ProcessingMode.Canceled;
                    return;
                }

                this.ProcessingMode = ProcessingMode.Completed;
            }
        }
        #endregion

        #region CheckForUpdate
        /// <summary>
        /// System.Deployment.Application.UpdateLocation `FbNA
        /// VXV𗘗pł邩ǂmF܂B
        /// </summary>
        /// <returns>
        /// VXV𗘗płꍇ trueBȊȌꍇ falseB
        /// </returns>
        /// <exception cref="System.Deployment.Application.InvalidDeploymentException">
        /// zu}jtFXgjĂ܂B
        /// ̖ɂ́AAvP[VĔzuKvƎv܂B
        /// ̗ÓASystem.Deployment.Application.CheckForUpdateCompleted Cxg 
        /// System.ComponentModel.AsyncCompletedEventArgs.Error vpeBŊmFł܂B
        /// </exception>
        /// <exception cref="System.Deployment.Application.DeploymentDownloadException">
        /// zu}jtFXg_E[hł܂B
        /// ̗ÓASystem.Deployment.Application.CheckForUpdateCompleted Cxg 
        /// System.ComponentModel.AsyncCompletedEventArgs.Error vpeBŊmFł܂B
        /// </exception>
        /// <exception cref="System.InvalidOperationException">
        /// XVɎsłƂɂ̃\bhĂяoƁAClickOnce ͑ɂ̗OX[܂B
        /// </exception>
        public bool CheckForUpdate()
        {
            this.ProcessingMode = ProcessingMode.NoAction;
            
            bool isDialogOpener = EnterClickOnceDialog();

            try
            {
                using (ClickOnceLog log = new ClickOnceLog())
                {
                    currentLogFilePath = log.LogFilePath;

                    currentUpdateCheckInfo.InitializeData();

                    ProcessingMode = ProcessingMode.Processing;
                    clickOnceDialog.IsCanceled = false;

                    appDeployment.CheckForUpdateAsync();

                    if (WaitAsyncMethod() == false)
                    {
                        return false;
                    } 
                }
            }
            finally
            {
                ExitClickOnceDialog(isDialogOpener);
            }

            if (currentUpdateCheckInfo.UpdateAvailable)
            {
                return true;
            }
            return false;
        }

        void appDeployment_CheckForUpdateProgressChanged(object sender, DeploymentProgressChangedEventArgs e)
        {
            lock (lockObject)
            {
                if (clickOnceDialog.IsCanceled)
                {
                    appDeployment.CheckForUpdateAsyncCancel();
                    clickOnceDialog.IsCanceled = false;
                }
            }
        }

        void appDeployment_CheckForUpdateCompleted(object sender, CheckForUpdateCompletedEventArgs e)
        {
            lock (lockObject)
            {
                if (e.Error != null)
                {
                    ChangeToErrorPage("XV̊mFɎs܂B", e.Error.Message);
                    return;
                }
                else if (e.Cancelled == true)
                {
                    this.ProcessingMode = ProcessingMode.Canceled;
                    return;
                }

                currentUpdateCheckInfo.UpdateAvailable = e.UpdateAvailable;
                if (currentUpdateCheckInfo.UpdateAvailable)
                {
                    currentUpdateCheckInfo.AvailableVersion = e.AvailableVersion;
                    currentUpdateCheckInfo.IsUpdateRequired = e.IsUpdateRequired;
                    currentUpdateCheckInfo.MinimumRequiredVersion = e.MinimumRequiredVersion;
                    currentUpdateCheckInfo.UpdateSizeBytes = e.UpdateSizeBytes;
                }
                this.ProcessingMode = ProcessingMode.Completed;
            }
        }
        #endregion

        #region GetDepolymentManifestInfo\bh̏
        /// <summary>
        /// System.Deployment.Application.UpdateLocation `FbNA
        /// XVɊւe擾܂B
        /// </summary>
        /// <returns>
        /// XVɊւe擾łꍇ trueBȊȌꍇ falseB
        /// </returns>
        private bool GetDepolymentManifestInfo()
        {
            lock (lockObject)
            {
                this.ProcessingMode = ProcessingMode.NoAction;

                bool isDialogOpener = EnterClickOnceDialog();

                // ClickOnce̋@\ł͂Ȃ̂ŃO t@C͍쐬ȂB
                currentLogFilePath = "";

                currentPublisherName = "";
                currentProductName = "";
                currentSupportUrl = "";
                currentServerName = "";
                currentIconFilePath = "";
                currentInstallMode = false;

                this.ProcessingMode = ProcessingMode.Processing;
                clickOnceDialog.IsCanceled = false;
                
                try
                {
                    string dotApplicationFile = appDeployment.UpdateLocation.AbsoluteUri;
                    if (dotApplicationFile.Length < 3)
                    {
                        return false;
                    }
                    if (dotApplicationFile.Substring(0, 2) == @"\\")
                    {
                        int lastPos = dotApplicationFile.LastIndexOf(@"\");
                        currentServerName = dotApplicationFile.Substring(0, lastPos);
                    }
                    else
                    {
                        int firstPos = dotApplicationFile.IndexOf(@"://");
                        firstPos += 3;
                        int lastPos = dotApplicationFile.IndexOf(@"/", firstPos);
                        currentServerName = dotApplicationFile.Substring(firstPos, lastPos - firstPos);
                    }

                    Thread worker = new Thread(new ParameterizedThreadStart(GetDepolymentManifestInfo_Worker));
                    worker.Start(dotApplicationFile);

                    if (WaitAsyncMethod(ref worker) == false)
                    {
                        return false;
                    }
                    
                    this.ProcessingMode = ProcessingMode.Completed;
                }
                catch
                {
                    this.ProcessingMode = ProcessingMode.Error;
                }
                finally
                {
                    ExitClickOnceDialog(isDialogOpener);
                }

                if (this.ProcessingMode == ProcessingMode.Completed)
                {
                    return true;
                }
            }
            return false;
        }

        // ʃXbhŎs郁\bh
        private static void GetDepolymentManifestInfo_Worker(object updateLocationObject)
        {
            ClickOnceAppDeployment deployment = ClickOnceAppDeployment.CurrentDeployment;
            try
            {
                string dotApplicationFile = updateLocationObject as string;

                XmlDocument doc = new XmlDocument();
                doc.Load(dotApplicationFile);

                XmlNamespaceManager nsmgrAsmv1 = new XmlNamespaceManager(doc.NameTable);
                nsmgrAsmv1.AddNamespace("asmv1", "urn:schemas-microsoft-com:asm.v1");
                XmlNode nodeAssembly = doc.DocumentElement.SelectSingleNode("/asmv1:assembly", nsmgrAsmv1);
                XmlNode nodeDescription = nodeAssembly["description"];
                deployment.currentProductName = nodeDescription.Attributes["asmv2:product"].InnerText;
                XmlNode attributeSupportUrl = nodeDescription.Attributes["asmv2:supportUrl"];
                if (attributeSupportUrl != null)
                {
                    deployment.currentSupportUrl = attributeSupportUrl.InnerText;
                }
                deployment.currentPublisherName = nodeDescription.Attributes["asmv2:publisher"].InnerText;
               
                bool mapFileExtentions = false;
                XmlNode nodeDeployment = nodeAssembly["deployment"];
                XmlAttribute attributeMapFileExtentions = nodeDeployment.Attributes["mapFileExtensions"];
                if (attributeMapFileExtentions != null)
                {
                    string mapFileExtentionsText = attributeMapFileExtentions.InnerText.ToLower();
                    if (mapFileExtentionsText == "true")
                    {
                        mapFileExtentions = true;
                    }
                }
                XmlAttribute attributeInstallMode = nodeDeployment.Attributes["install"];
                if (attributeInstallMode != null)
                {
                    string installModeText = attributeInstallMode.InnerText.ToLower();
                    if (installModeText == "true")
                    {
                        deployment.currentInstallMode = true;
                    }
                }
                
                XmlNode nodeDependency = nodeAssembly["dependency"]["dependentAssembly"];
                int lastPos = dotApplicationFile.LastIndexOf(@"/");
                string currentRootPath = dotApplicationFile.Substring(0, lastPos + 1);
                string dotManifestFile = currentRootPath + nodeDependency.Attributes["codebase"].InnerText;
                dotManifestFile = dotManifestFile.Replace('\\', '/');

                XmlDocument docManifest = new XmlDocument();
                docManifest.Load(dotManifestFile);

                XmlNode nodeManifestAssembly = docManifest.DocumentElement.SelectSingleNode("/asmv1:assembly", nsmgrAsmv1);
                XmlNode nodeManifestDescription = nodeManifestAssembly["description"];
                if (nodeManifestDescription != null)
                {
                    XmlAttribute attributeIconFile = nodeManifestDescription.Attributes["asmv2:iconFile"];
                    if (attributeIconFile != null)
                    {
                        int appLastPos = dotManifestFile.LastIndexOf(@"/");
                        string currentAppPath = dotManifestFile.Substring(0, appLastPos + 1);
                        string currentIconFilePath = currentAppPath + attributeIconFile.InnerText;
                        if (mapFileExtentions)
                        {
                            currentIconFilePath += ".deploy";
                        }
                        deployment.currentIconFilePath = currentIconFilePath;
                    }
                }

                deployment.ProcessingMode = ProcessingMode.Completed;
            }
            catch
            {
                deployment.ProcessingMode = ProcessingMode.Error;
            }
        }
        #endregion

        #region Update\bh̏
        /// <summary>
        /// ̃AvP[V̍ŐVo[W̃_E[hƃCXg[Jn܂B
        /// </summary>
        /// <returns>
        /// AvP[VXVꂽꍇ trueBȊȌꍇ falseB
        /// </returns>
        /// <exception cref="System.Deployment.Application.DeploymentDownloadException">
        /// Vzulbg[N̏ꏊ_E[hł܂łB
        /// </exception>
        /// <exception cref="System.Deployment.Application.TrustNotGrantedException">
        /// [J Rs[^́ÃAvP[Vs邽߂ɗvANZXxt^܂łB
        /// </exception>
        /// <exception cref="System.Deployment.Application.InvalidDeploymentException">
        /// ClickOnce ̔zujĂ܂B̖ffA@̃qgɂẮA
        /// uTroubleshooting ClickOnce DeploymentsvQƂĂB
        /// </exception>        
        public bool Update()
        {
            return Update(DeploymentMode.SilentUpdate);
        }

        public bool Update(bool isRequiredUpdate)
        {
            if (isRequiredUpdate == true)
            {
                return Update(DeploymentMode.RequiredUpdate);
            }
            else
            {
                return Update(DeploymentMode.NormalUpdate);
            }
        }

        private bool Update(DeploymentMode mode)
        {
            this.ProcessingMode = ProcessingMode.NoAction;

            bool isDialogOpener = EnterClickOnceDialog();

            try
            {
                // GetDepolymentManifestInfo\bh͓Ǝ̂߃O𐶐ȂȂB
                // ]ł̓G[͍sȂBG[ׂ͂Ĉȍ~ClickOnce APIŔB
                GetDepolymentManifestInfo();
                clickOnceDialog.PublisherName = currentPublisherName;
                clickOnceDialog.AppProductName = currentProductName;
                clickOnceDialog.SupportUrl = currentSupportUrl;
                clickOnceDialog.ServerName = currentServerName;
                clickOnceDialog.IconFilePath = currentIconFilePath;
                clickOnceDialog.HeaderTitle = DeploymentUtil.FormatHeaderTitleText(mode, currentProductName);
                clickOnceDialog.HeaderMessage = DeploymentUtil.FormatHeaderMessageText(mode);                
                clickOnceDialog.DeploymentMode = mode;

                if (mode == DeploymentMode.NormalUpdate)
                {
                    ChangeToCheckPane();

                    if (WaitConfirmation() == false)
                    {
                        return false;
                    }
                }

                using (ClickOnceLog log = new ClickOnceLog())
                {
                    currentLogFilePath = log.LogFilePath;

                    currentUpdateCheckInfo.InitializeData();

                    ProcessingMode = ProcessingMode.Processing;
                    clickOnceDialog.IsCanceled = false;

                    ChanteToUpdatingPage();

                    appDeployment.UpdateAsync();

                    if (WaitAsyncMethod() == false)
                    {
                        return false;
                    }
                }
            }
            finally
            {
                CompleteProgressMessage();

                ExitClickOnceDialog(isDialogOpener);
            }

            if (ProcessingMode == ProcessingMode.Completed)
            {
                return true;
            }

            return false;
        }

        void appDeployment_UpdateProgressChanged(object sender, DeploymentProgressChangedEventArgs e)
        {
            lock (lockObject)
            {
                if (clickOnceDialog.IsCanceled)
                {
                    appDeployment.UpdateAsyncCancel();
                    clickOnceDialog.IsCanceled = false;
                }

                UpdateProgressMessage(e.State, e.ProgressPercentage, e.BytesCompleted, e.BytesTotal, clickOnceDialog.DeploymentMode);
            }
        }

        void appDeployment_UpdateCompleted(object sender, AsyncCompletedEventArgs e)
        {
            lock (lockObject)
            {
                if (e.Error != null)
                {
                    ChangeToErrorPage("XVɎs܂B", e.Error.Message);
                    return;
                }
                else if (e.Cancelled == true)
                {
                    this.ProcessingMode = ProcessingMode.Canceled;
                    return;
                }

                this.ProcessingMode = ProcessingMode.Completed;
            }
        }
        #endregion

        #region OɏJvpeBQ
        /// <summary>
        /// AvP[V̔zu}jtFXgN邽߂Ɏgp URL 擾܂B
        /// </summary>
        /// <value>
        /// zu}jtFXg TrustUrlParameters vpeB false ̏ꍇA
        /// ܂̓[U[ UNC w肵ĔzuJĂ邩Azu[JŊJĂꍇ́A 0 ̕B
        /// ȊȌꍇ́AׂẴp[^܂ށAAvP[V̋NɎgp銮S URLB 
        /// </value>
        public Uri ActivationUri
        {
            get { return appDeployment.ActivationUri; }
        }
        
        /// <summary>
        /// ̔zǔ݂ ApplicationDeployment IuWFNg Ԃ܂B
        /// </summary>
        /// <value>
        /// ݂̔zuB
        /// </value>
        /// <exception cref="System.Deployment.Application.InvalidDeploymentException">
        /// ClickOnce ȊÕAvP[V炱̐ÓIvpeBĂяoƂ܂B
        /// </exception>
        public static ClickOnceAppDeployment CurrentDeployment
        {
            get
            {
                if (currentClickOnceAppDeployment == null)
                {
                    currentClickOnceAppDeployment = new ClickOnceAppDeployment();
                }
                return currentClickOnceAppDeployment;
            }
        }

        /// <summary>
        /// AvP[V̌ݎs̃CX^X̔zũo[W擾܂B
        /// </summary>
        /// <value>
        /// ݂̔zũo[WB
        /// </value>
        public Version CurrentVersion
        {
            get { return appDeployment.CurrentVersion; }
        
        }

        /// <summary>
        /// ClickOnce f[^ fBNgւ̃pX擾܂B
        /// </summary>
        /// <value>
        /// [J fBXÑAvP[Ṽf[^ fBNgւ̃pX܂ޕB 
        /// </value>
        public string DataDirectory
        {
            get { return appDeployment.DataDirectory; }
        }
        
        /// <summary>
        /// NCAg Rs[^ł̃AvP[Vŝ͍񂪏߂Ăǂl擾܂B
        /// </summary>
        /// <value>
        /// NCAg Rs[^ŁA
        /// AvP[V̂̃o[W܂ňxsĂȂꍇ trueBȊȌꍇ falseB
        /// </value>
        public bool IsFirstRun
        {
            get { return appDeployment.IsFirstRun; }
        }

        /// <summary>
        /// ݂̃AvP[V ndptecclick AvP[Vǂl擾܂B
        /// </summary>
        /// <returns>
        /// ̃AvP[V ndptecclick AvP[V̏ꍇ trueBȊȌꍇ falseB
        /// </returns>
        public static bool IsNetworkDeployed
        {
            get { return ApplicationDeployment.IsNetworkDeployed; }
        }

        /// <summary>
        /// ndptecclick AvP[V̍XVŌɃ`FbN擾܂B
        /// </summary>
        /// <returns>
        /// ŏIXV`FbN System.DateTimeB
        /// </returns>
        public DateTime TimeOfLastUpdateCheck
        {
            get { return appDeployment.TimeOfLastUpdateCheck; }
        }

        /// <summary>
        /// AvP[V̍XVɁAAvP[V̊S擾܂B
        /// </summary>
        public string UpdatedApplicationFullName
        {
            get { return appDeployment.UpdatedApplicationFullName; }
        }
        
        /// <summary>
        /// ŋ߃_E[hꂽXṼo[W擾܂B
        /// </summary>
        /// <value>
        /// XṼo[W System.VersionB
        /// </value>
        public Version UpdatedVersion
        {
            get { return appDeployment.UpdatedVersion; }
        }
        
        /// <summary>
        /// ̃AvP[V̍XVł Web TCg܂̓t@CL擾܂B
        /// </summary>
        /// <value>
        /// HTTPAHTTPSA܂̓t@C URL Ƃĕ\XṼpXB
        /// ܂ Windows lbg[N t@C pX (UNC) Ƃĕ\XṼpXB
        /// </value>
        public Uri UpdateLocation
        {
            get { return appDeployment.UpdateLocation; }
        }

        /// <summary>
        /// Abvf[gK{ǂ擾܂B
        /// </summary>
        public bool IsUpdateRequired
        {
            get
            {
                if (currentUpdateCheckInfo.IsMemberFilled == false)
                {
                    throw new MemberAccessException("IsUpdateRequiredvpeB𗘗pɂ́AOCheckForUpdate\bhĂяoKv܂B");                    
                }
                return currentUpdateCheckInfo.IsUpdateRequired;
            }
        }

        ///// <summary>
        ///// O t@C̏ꏊ擾܂B
        ///// </summary>
        //public string LogFilePath
        //{
        //    get { return currentLogFilePath; }
        //}

        ///// <summary>
        ///// s҂̖O擾܂B
        ///// </summary>
        //public string PublisherName
        //{
        //    get { return currentPublisherName; }
        //}

        ///// <summary>
        ///// i̖O擾܂B
        ///// </summary>
        //public string ProductName
        //{
        //    get { return currentProductName; }
        //}

        ///// <summary>
        ///// T|[gURL擾܂B
        ///// </summary>
        //public string SupportUrl
        //{
        //    get { return currentSupportUrl; }
        //}

        ///// <summary>
        ///// zM̖O擾܂B
        ///// </summary>
        //public string ServerName
        //{
        //    get { return currentServerName; }
        //}

        ///// <summary>
        ///// zM̃ACR擾܂B
        ///// </summary>
        //public string IconFilePath
        //{
        //    get { return currentIconFilePath; }
        //}

        /// <summary>
        /// 
        /// </summary>
        /// <value>
        /// 
        /// </value>
        public string SystemDeploymentLoction
        {
            get
            {
                Assembly asm = DeploymentUtil.GetSystemDeploymentAssembly();
                return asm.Location;
            }
        }

        /// <summary>
        /// 
        /// </summary>
        /// <value>
        /// 
        /// </value>
        public string SystemDeploymentVersion
        {
            get
            {
                Assembly asm = DeploymentUtil.GetSystemDeploymentAssembly();
                return asm.ImageRuntimeVersion;
            }
        }

        /// <summary>
        /// 
        /// </summary>
        /// <value>
        /// 
        /// </value>
        public string SystemDeploymentFullName
        {
            get
            {
                Assembly asm = DeploymentUtil.GetSystemDeploymentAssembly();
                return asm.FullName;
            }
        }

        public ProcessingMode ProcessingMode
        {
            get { return currentProcessingMode; }
            set { currentProcessingMode = value; }
        }
        #endregion

        #region ClickOnceDownloadDialogp̃\bhQ
        /// <summary>
        /// 
        /// </summary>
        /// <returns></returns>
        private bool EnterClickOnceDialog()
        {
            bool isDialogOpener = showingClickOnceDialog;
            if (isDialogOpener == false)
            {
                showingClickOnceDialog = true;
                ShowClickOnceDialog();
            }
            else
            {
                ChangeToConnectingPage();
            }
            return isDialogOpener;
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="isDialogOpener"></param>
        private void ExitClickOnceDialog(bool isDialogOpener)
        {
            if (isDialogOpener == false)
            {
                CloseClickOnceDialog();
                showingClickOnceDialog = false;
            }
        }

        /// <summary>
        /// ClickOnce_E[hp̃_CAO\܂B
        /// </summary>
        /// <param name="mode"></param>
        private void ShowClickOnceDialog()
        {
            DisableAllForms();

            if (clickOnceDialog == null)
            {
                Form currentActiveForm = Form.ActiveForm;
                clickOnceDialog = new ClickOnceDownloadDialog(currentActiveForm);
            }
            ChangeToConnectingPage();
            clickOnceDialog.Show();
        }

        /// <summary>
        /// ClickOnce_CAO̐ڑy[W\܂B
        /// </summary>
        private void ChangeToConnectingPage()
        {
            clickOnceDialog.Pane = PaneMode.Connecting;
        }

        /// <summary>
        /// ClickOnce_CAO̊mFy[W\܂B
        /// </summary>
        private void ChangeToCheckPane()
        {
            clickOnceDialog.Pane = PaneMode.Check;
        }

        /// <summary>
        /// ClickOnce_CAO̍XVy[W\܂B
        /// </summary>
        private void ChanteToUpdatingPage()
        {
            clickOnceDialog.Pane = PaneMode.Updating;
        }

        /// <summary>
        /// ClickOnce_CAÕG[y[W\܂B
        /// </summary>
        /// <param name="mode"></param>
        private void ChangeToErrorPage(string errorTitle, string errorMessage)
        {
            clickOnceDialog.LogFilePath = currentLogFilePath;
            clickOnceDialog.ErrorTitle = errorTitle;
            clickOnceDialog.ErrorMessage = errorMessage;
            clickOnceDialog.Pane = PaneMode.Error;
        }

        /// <summary>
        /// ClickOnce_E[hp̃_CAO܂B
        /// </summary>
        private void CloseClickOnceDialog()
        {
            EnableAllForms();

            Form currentActiveForm = clickOnceDialog.PreviousActiveForm;
            clickOnceDialog.Close();
            currentActiveForm.Activate();
            clickOnceDialog.Dispose();
            clickOnceDialog = null;
        }

        /// <summary>
        /// ׂẮiLȁjtH[𖳌܂B
        /// </summary>
        private void DisableAllForms()
        {
            foreach (Form oneForm in Application.OpenForms)
            {
                if (oneForm.Enabled == true)
                {
                    oneForm.Enabled = false;
                    changedForms.Add(oneForm);
                }
            }
        }

        /// <summary>
        /// ɂׂẴtH[ĂїL܂B
        /// </summary>
        private void EnableAllForms()
        {
            foreach (Form oneForm in changedForms)
            {
                oneForm.Enabled = true;
            }
            changedForms.Clear();
        }
        #endregion

        #region "ClickOncesp̃\bhQ"
        private void UpdateProgressMessage(DeploymentProgressState state, int percentage, long bytesCompleted, long bytesTotal, DeploymentMode mode)
        {
            if (clickOnceDialog.IsDisposed == false)
            {
                clickOnceDialog.SuspendLayout();
                if (state == DeploymentProgressState.DownloadingApplicationFiles)
                {
                    if ((percentage == 100) && (bytesCompleted == bytesTotal))
                    {
                        clickOnceDialog.ProgressMessageText = "AvP[V̏...";
                        if (clickOnceDialog.Enabled == true)
                        {
                            clickOnceDialog.Enabled = false;
                        }
                    }
                    else
                    {
                        if (clickOnceDialog.Enabled == false)
                        {
                            clickOnceDialog.Enabled = true;
                        }
                        clickOnceDialog.ProgressMessageText = DeploymentUtil.FormatProgressMessageText(bytesCompleted, bytesTotal);
                    }
                    clickOnceDialog.ProgressTitleText = DeploymentUtil.FormatProgressTitleText(mode, percentage, currentProductName);
                    clickOnceDialog.ProgressPercentage = percentage;
                }
                else
                {
                    clickOnceDialog.ProgressMessageText = DeploymentUtil.FormatProgressMessageText(0, 0);
                    clickOnceDialog.ProgressTitleText = DeploymentUtil.FormatProgressTitleText(mode, 0, currentProductName);
                    clickOnceDialog.ProgressPercentage = 0;
                }
                clickOnceDialog.ResumeLayout(false);
            }
        }

        private void CompleteProgressMessage()
        {
            if (clickOnceDialog.IsDisposed == false)
            {
                if (clickOnceDialog.Enabled == false)
                {
                    clickOnceDialog.Enabled = true;
                }
            }
            clickOnceDialog.DeploymentMode = DeploymentMode.None;
        }

        private bool WaitAsyncMethod()
        {
            bool wait = true;
            while (wait)
            {
                Thread.Sleep(100);
                Application.DoEvents();
                switch (this.ProcessingMode)
                {
                    case ProcessingMode.Processing:
                        if (clickOnceDialog.IsError)
                        {
                            this.ProcessingMode = ProcessingMode.Error;
                            return false;
                        }
                        break;
                    case ProcessingMode.NoAction:
                    case ProcessingMode.Error:
                    case ProcessingMode.Canceled:
                    case ProcessingMode.Skipped:
                        return false;
                    case ProcessingMode.Completed:
                        wait = false;
                        break;
                }
            }
            return true;
        }

        private bool WaitAsyncMethod(ref Thread worker)
        {
            bool wait = true;
            while (wait)
            {
                Thread.Sleep(100);
                Application.DoEvents();
                wait = worker.IsAlive;
                switch (this.ProcessingMode)
                {
                    case ProcessingMode.Processing:
                        break;
                    case ProcessingMode.NoAction:
                    case ProcessingMode.Error:
                    case ProcessingMode.Canceled:
                    case ProcessingMode.Skipped:
                        return false;
                    case ProcessingMode.Completed:
                        wait = false;
                        break;
                }
            }
            return true;
        }

        private bool WaitConfirmation()
        {
            while (true)
            {
                Thread.Sleep(100);
                Application.DoEvents();
                if (clickOnceDialog.IsCanceled)
                {
                    this.ProcessingMode = ProcessingMode.Skipped;
                    return false;
                }
                if (clickOnceDialog.IsProceeded)
                {
                    return true;
                }
            }
        }
        #endregion
    }

    public enum ProcessingMode
    {
        Error = -1,
        NoAction = 0,
        Processing = 1,
        Completed = 2,
        Canceled = 3,
        Skipped = 4
    }

    internal enum DeploymentMode
    {
        None = 0,
        Install = 1,
        Download = 2,
        NormalUpdate = 3,
        RequiredUpdate = 4,
        SilentUpdate = 5
    }

}

