using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Reflection;
using System.IO;
using System.Runtime.InteropServices;

namespace DigitaAdvantage.ClickOnce.Forms
{
    internal partial class ClickOnceDownloadDialog : Form
    {
        private MainPane currentMainPane;
        private Form parentMainForm;
        private PaneMode currentPaneFlag;

        private string currentLogFilePath;
        private string currentPublisherName;
        private string currentProductName;
        private string currentSupportUrl;
        private string currentServerName;
        private string currentIconFilePath;
        private string currentHeaderTitle;
        private string currentHeaderMessage;
        private string currentErrorMessage;
        private string currentErrorTitle;
        private DeploymentMode currentDeploymentMode;

        private const string undefinedPublisherName = "";
        private const string undefinedProductName = "";
        private const string undefinedServerName = "";

        public Form PreviousActiveForm
        {
            get { return parentMainForm; }
        }

        public string ProgressMessageText
        {
            set { if (currentPaneFlag == PaneMode.Updating) currentMainPane.ProgressMessageText = value; }
        }

        public string ProgressTitleText
        {
            set { this.Text = value; }
        }

        public int ProgressPercentage
        {
            set { if (currentPaneFlag == PaneMode.Updating) currentMainPane.ProgressPercentage = value; }
        }

        public string LogFilePath
        {
            set { currentLogFilePath = value;  }
            get { return currentLogFilePath; }
        }

        public string PublisherName
        {
            get { return currentPublisherName; }
            set { currentPublisherName = value; }
        }

        public string AppProductName
        {
            get { return currentProductName; }
            set { currentProductName = value; }
        }

        public string SupportUrl
        {
            get { return currentSupportUrl; }
            set { currentSupportUrl = value; }
        }

        public string ServerName
        {
            get { return currentServerName; }
            set { currentServerName = value; }
        }

        public string IconFilePath
        {
            get { return currentIconFilePath; }
            set { currentIconFilePath = value; }
        }        

        public string HeaderTitle
        {
            get { return currentHeaderTitle; }
            set { currentHeaderTitle = value; }
        }

        public string HeaderMessage
        {
            get { return currentHeaderMessage; }
            set { currentHeaderMessage = value; }
        }

        public string ErrorMessage
        {
            get { return currentErrorMessage; }
            set { currentErrorMessage = value; }
        }

        public string ErrorTitle
        {
            get { return currentErrorTitle; }
            set { currentErrorTitle = value; }
        }

        public DeploymentMode DeploymentMode
        {
            get { return currentDeploymentMode; }
            set { currentDeploymentMode = value; }
        }

        public bool IsCanceled
        {
            get { return currentMainPane.IsCanceled; }
            set { currentMainPane.IsCanceled = value; }
        }

        public bool IsProceeded
        {
            get { return currentMainPane.IsProceeded; }
            set { currentMainPane.IsProceeded = value; }
        }

        public bool IsError
        {
            get { return currentMainPane.IsError; }
            set { currentMainPane.IsError = value; }
        }

        public ClickOnceDownloadDialog(Form parentForm)
        {
            parentMainForm = parentForm;
            currentLogFilePath = "";
            currentPublisherName = "";
            currentProductName = "";
            currentSupportUrl = "";
            currentServerName = "";
            currentIconFilePath = "";
            currentHeaderTitle = "";
            currentHeaderMessage = "";
            currentErrorMessage = "";
            currentErrorTitle = "";
            currentDeploymentMode = DeploymentMode.None;

            Assembly asm = DeploymentUtil.GetSystemDeploymentAssembly();
            Stream streamAppIco = asm.GetManifestResourceStream("form.ico");

            InitializeComponent();

            if (streamAppIco != null)
            {
                try
                {
                    Icon normalIcon = new Icon(streamAppIco, SystemInformation.IconSize);
                    if (normalIcon != null)
                    {
                        this.Icon = normalIcon;
                    }
                }
                catch
                {
                }
            }

            if (parentMainForm != null)
            {
                this.Location = new Point(parentMainForm.Left + 10, parentMainForm.Top + 10);
            }
            else
            {
                this.StartPosition = System.Windows.Forms.FormStartPosition.WindowsDefaultLocation;
            }

            currentPaneFlag = PaneMode.Connecting;
        }

        private void ClickOnceDownloadDialog_FormClosing(object sender, FormClosingEventArgs e)
        {
            base.OnClosing(e);

            if (currentMainPane.OnClosing())
            {
                e.Cancel = true;
                base.Hide();
            }
            else
            {
                e.Cancel = true;
            }
        }

        protected override void OnVisibleChanged(EventArgs e)
        {
            base.OnVisibleChanged(e);

            if (this.Visible && (this != Form.ActiveForm))
            {
                this.Activate();
            }
        }

        public PaneMode Pane
        {
            get
            {
                return currentPaneFlag;
            }
            set
            {
                currentPaneFlag = value;

                this.SuspendLayout();
                if (this.currentMainPane != null)
                {
                    this.currentMainPane.Dispose();
                    this.currentMainPane = null;
                }

                switch (currentPaneFlag)
                {
                    case PaneMode.Connecting:
                        this.currentMainPane = new ConnectingPane(this);
                        break;
                    case PaneMode.Check:
                        this.currentMainPane = new CheckPane(this, currentProductName, currentServerName, currentSupportUrl);
                        break;
                    case PaneMode.Updating:
                        this.currentMainPane = new UpdatingPane(this, currentPublisherName, currentProductName, currentServerName, currentSupportUrl, currentHeaderTitle, currentHeaderMessage, currentIconFilePath);
                        break;
                    case PaneMode.Error:
                        this.currentMainPane = new ErrorPane(this, currentErrorTitle, currentErrorMessage, LogFilePath);
                        break;
                }

                this.currentMainPane.Dock = DockStyle.Fill;
                this.ClientSize = this.currentMainPane.ClientSize;
                this.Controls.Add(this.currentMainPane);
                this.Font = SystemFonts.MessageBoxFont;
                this.ResumeLayout(false);

            }
        }
    }

    public enum PaneMode
    {
        Connecting = 0,
        Check = 1,
        Updating = 2,
        Error = 3
    }
}