using System;
using System.Collections.Generic;
using System.Text;
using Microsoft.Win32;
using System.Security;
using System.IO;
using System.Diagnostics;
using System.Runtime.InteropServices;

namespace DigitaAdvantage.ClickOnce
{
    sealed class ClickOnceLog : IDisposable
    {
        private const string regKeyPath = @"SOFTWARE\Classes\Software\Microsoft\Windows\CurrentVersion\Deployment";
        private const string regValueName = "LogFilePath";
        private const string regExecutingMode = "ExecutingCustomLogMode";
        private const string prefixcustomLogUrl = "DigitaAdvantage_ClickOnce_CustomLog_";

        private string customLogUrl;

        private string savedLogFilePath;
        public string LogFilePath
        {
            get { return savedLogFilePath; }
            set { savedLogFilePath = value; }
        }

        private bool isExecutingCustomLogMode;
        public bool ExecutingCustomLogMode
        {
            get { return isExecutingCustomLogMode; }
            set { isExecutingCustomLogMode = value; }
        }

        public ClickOnceLog()
        {
            customLogUrl = prefixcustomLogUrl + DateTime.Now.Ticks.ToString();
            savedLogFilePath = null;
            isExecutingCustomLogMode = false;

            LogFilePath = GetLogFilePathFromRegistry();
            if (String.IsNullOrEmpty(LogFilePath))
            {
                string tempLogFilePath = GetLogFilePathFromWinINet(customLogUrl);
                if (String.IsNullOrEmpty(tempLogFilePath) == false)
                {
                    if (SetLogFilePathToRegistry(tempLogFilePath))
                    {
                        ExecutingCustomLogMode = true;
                        LogFilePath = tempLogFilePath;
                    }
                }
                else
                {
                    Debug.Assert(false, "O t@C pXł܂łB");
                }
            }
        }

        void IDisposable.Dispose()
        {
            if (ExecutingCustomLogMode)
            {
                CommitLogFilePathFromWinINet(customLogUrl, LogFilePath);
                DeleteLogFilePathFromRegistry();
            }
        }

        #region WXg

        private string GetLogFilePathFromRegistry()
        {
            string regLogFilePath = null;
            string executingModePath = null;

            try
            {
                using (RegistryKey regKey = Registry.CurrentUser.OpenSubKey(regKeyPath))
                {
                    if (regKey != null)
                    {
                        regLogFilePath = regKey.GetValue(regValueName) as string;
                        executingModePath = regKey.GetValue(regExecutingMode) as string;
                    }
                }
            }
            catch (ArgumentNullException)
            {
            }
            catch (ArgumentException)
            {
            }
            catch (ObjectDisposedException)
            {
            }
            catch (SecurityException)
            {
            }
            catch (IOException)
            {
            }
            catch (UnauthorizedAccessException)
            {
            }

            if ((executingModePath != null) && (regLogFilePath == executingModePath))
            {
                return null;
            }

            return regLogFilePath;
        }

        private bool SetLogFilePathToRegistry(string customLogFilePath)
        {
            try
            {
                using (RegistryKey regKey = Registry.CurrentUser.CreateSubKey(regKeyPath))
                {
                    if (regKey != null)
                    {
                        regKey.SetValue(regValueName, customLogFilePath);
                        regKey.SetValue(regExecutingMode, customLogFilePath);
                    }
                }
            }
            catch (ArgumentNullException)
            {
                return false;
            }
            catch (ArgumentException)
            {
                return false;
            }
            catch (ObjectDisposedException)
            {
                return false;
            }
            catch (SecurityException)
            {
                return false;
            }
            catch (IOException)
            {
                return false;
            }
            catch (UnauthorizedAccessException)
            {
                return false;
            }

            return true;
        }

        private bool DeleteLogFilePathFromRegistry()
        {
            try
            {
                using (RegistryKey regKey = Registry.CurrentUser.CreateSubKey(regKeyPath))
                {
                    if (regKey != null)
                    {
                        regKey.DeleteValue(regValueName, false);
                        regKey.DeleteValue(regExecutingMode, false);
                    }
                }
            }
            catch (ArgumentNullException)
            {
                return false;
            }
            catch (ArgumentException)
            {
                return false;
            }
            catch (ObjectDisposedException)
            {
                return false;
            }
            catch (SecurityException)
            {
                return false;
            }
            catch (UnauthorizedAccessException)
            {
                return false;
            }

            return true;
        }

        #endregion

        #region WinINet

        public string GetLogFilePathFromWinINet(string urlName)
        {
            string inetLogFilePath = null;

            try
            {
                StringBuilder filePathBuilder = new StringBuilder(261);
                if (!CreateUrlCacheEntry(urlName, 0, "log", filePathBuilder, 0))
                {
                    return null;
                }
                inetLogFilePath = filePathBuilder.ToString();
            }
            catch (ArgumentNullException)
            {
            }
            catch (ArgumentException)
            {
            }
            catch (ObjectDisposedException)
            {
            }
            catch (SecurityException)
            {
            }
            catch (IOException)
            {
            }
            catch (UnauthorizedAccessException)
            {
            }

            return inetLogFilePath;
        }

        public void CommitLogFilePathFromWinINet(string urlName, string logFilePath)
        {
            // CommitUrlCacheEntry֐STICKY_CACHE_ENTRYw肷
            // W10ԃɃLbVc葱܂B
            // iCacheEntryType = 4 :STICKY_CACHE_ENTRYj
            CommitUrlCacheEntry(urlName, logFilePath, (long)0, (long)0, 4, null, 0, null, null);
        }

        [DllImport("wininet.dll")]
        private extern static bool CreateUrlCacheEntry(
            string lpszUrlName, int dwExpectedFileSize,
            string lpszFileExtension, StringBuilder lpszFileName, int dwReserved);
        [DllImport("wininet.dll")]
        private extern static bool CommitUrlCacheEntry(
            string lpszUrlName, string lpszLocalFileName,
            long ExpireTime, long LastModifiedTime, uint CacheEntryType,
            string lpHeaderInfo, int dwHeaderSize, string lpszFileExtension, string lpszOriginalUrl);

        #endregion

    }
}
