using System;
using System.Collections.Generic;
using System.Text;
using System.Reflection;
using System.IO;
using System.Drawing;
using System.Deployment.Application;

namespace DigitaAdvantage.ClickOnce
{
    internal static class DeploymentUtil
    {
        private static Assembly savedAssembly = null;

        public static Assembly GetSystemDeploymentAssembly()
        {
            if (savedAssembly == null)
            {
                Type t = typeof(ApplicationDeployment);
                string asmFullName = t.Assembly.FullName.ToString();
                if (asmFullName.Length <= 0)
                {
                    asmFullName = "System.Deployment, Version=2.0.0.0, Culture=neutral, PublicKeyToken=b03f5f7f11d50a3a";
                }
                savedAssembly = Assembly.Load(asmFullName);
            }
            return savedAssembly;
        }

        public static string GetSystemDeploymentLoction()
        {
            Assembly asm = GetSystemDeploymentAssembly();
            return asm.Location;
        }

        public static string GetSystemDeploymentVersion()
        {
            Assembly asm = GetSystemDeploymentAssembly();
            return asm.ImageRuntimeVersion;
        }

        public static string GetSystemDeploymentFullName()
        {
            Assembly asm = GetSystemDeploymentAssembly();
            return asm.FullName;
        }

        private const long JUST_1KB     = 0x400L;           // 1024
        private const long JUST_10KB    = 0x2800L;          // 10240
        private const long JUST_100KB   = 0x19000L;         // 102400
        private const long JUST_1MB     = 0x100000L;        // 1048576i10242j
        private const long JUST_10MB    = 0xa00000L;        // 10485760
        private const long JUST_100MB   = 0x6400000L;       // 104857600
        private const long JUST_1GB     = 0x40000000L;      // 1073741824i10243j
        private const long JUST_10GB    = 0x280000000L;     // 10737418240
        private const long JUST_100GB   = 0x1900000000L;    // 107374182400

        public static string FormatProgressBytesText(long bytes)
        {
            string formatText = "";
            float formatBytes = 0.0F;
            if (bytes < JUST_1KB)
            {
                formatText = "{0} oCg";
                formatBytes = (float)bytes;
            }
            else if ((JUST_1KB <= bytes) && (bytes < JUST_10KB))
            {
                formatText = "{0:f2} KB";
                formatBytes = (float)bytes / (float)JUST_1KB;
            }
            else if ((JUST_10KB <= bytes) && (bytes < JUST_100KB))
            {
                formatText = "{0:f1} KB";
                formatBytes = (float)bytes / (float)JUST_1KB;
            }
            else if ((JUST_100KB <= bytes) && (bytes < JUST_1MB))
            {
                formatText = "{0:f0} KB";
                formatBytes = (float)bytes / (float)JUST_1KB;
            }
            else if ((JUST_1MB <= bytes) && (bytes < JUST_10MB))
            {
                formatText = "{0:f2} MB";
                formatBytes = (float)bytes / (float)JUST_1MB;
            }
            else if ((JUST_10MB <= bytes) && (bytes < JUST_100MB))
            {
                formatText = "{0:f1} MB";
                formatBytes = (float)bytes / (float)JUST_1MB;
            }
            else if ((JUST_100MB <= bytes) && (bytes < JUST_1GB))
            {
                formatText = "{0:f0} MB";
                formatBytes = (float)bytes / (float)JUST_1MB;
            }
            else if ((JUST_1GB <= bytes) && (bytes < JUST_10GB))
            {
                formatText = "{0:f2} GB";
                formatBytes = (float)bytes / (float)JUST_1GB;
            }
            else if ((JUST_10GB <= bytes) && (bytes < JUST_100GB))
            {
                formatText = "{0:f1} GB";
                formatBytes = (float)bytes / (float)JUST_1GB;
            }
            else if (JUST_100GB <= bytes)
            {
                formatText = "{0:f0} GB";
                formatBytes = (float)bytes / (float)JUST_1GB;
            }
            return string.Format(formatText, formatBytes);
        }

        public static string FormatProgressMessageText(long bytesCompleted, long bytesTotal)
        {
            return string.Format("_E[h: {0} / {1}",
                FormatProgressBytesText(bytesCompleted),
                FormatProgressBytesText(bytesTotal));
        }

        public static string FormatProgressTitleText(DeploymentMode mode, long progressPercentage, string appProductName)
        {
            if ((mode == DeploymentMode.NormalUpdate) || (mode == DeploymentMode.SilentUpdate))
            {
                return string.Format("({0}%) {1} XVĂ܂B", progressPercentage, appProductName);
            }
            else if (mode == DeploymentMode.RequiredUpdate)
            {
                return string.Format("({0}%)  {1} p̕KvȍXV", progressPercentage, appProductName);
            }
            else if (mode == DeploymentMode.Download)
            {
                return string.Format("({0}%) {1} _E[hĂ܂B", progressPercentage, appProductName);
            }

            // DeploymentMode.Intall or DeploymentMode.None
            return string.Format("({0}%) {1} CXg[Ă܂B", progressPercentage, appProductName);
        }

        public static string FormatHeaderTitleText(DeploymentMode mode, string appProductName)
        {
            if ((mode == DeploymentMode.NormalUpdate) || (mode == DeploymentMode.SilentUpdate))
            {
                return string.Format("{0} XVĂ܂B", appProductName);
            }
            else if (mode == DeploymentMode.RequiredUpdate)
            {
                return string.Format(" {0} p̕KvȍXV", appProductName);
            }
            else if (mode == DeploymentMode.Download)
            {
                return string.Format("{0} _E[hĂ܂B", appProductName);
            }

            // DeploymentMode.Intall or DeploymentMode.None
            return string.Format("{0} CXg[Ă܂B", appProductName);
        }

        public static string FormatHeaderMessageText(DeploymentMode mode)
        {
            string formatText = "ɂ͐邱Ƃ܂B" + 
                "R|[lg{0}̓Rs[^gđ̃^XNs邱Ƃł܂B";

            if ((mode == DeploymentMode.NormalUpdate) || (mode == DeploymentMode.SilentUpdate))
            {
                return string.Format(formatText, "XV");
            }
            else if (mode == DeploymentMode.RequiredUpdate)
            {
                return string.Format(formatText, "XV");
            }
            else if (mode == DeploymentMode.Download)
            {
                return string.Format(formatText, "_E[h"); 
            }

            // DeploymentMode.Intall or DeploymentMode.None
            return string.Format(formatText, "CXg[");
        }

        public static bool IsValidSupportUrl(string url)
        {
            if (url == null)
            {
                return false;
            }
            if (url.Length <= 0)
            {
                return false;
            }
            if ((url.StartsWith("http://") == false) && (url.ToUpper().StartsWith("https://") == false))
            {
                return false;
            }
            return true;
        }

        public static Bitmap GetAppIcon32x32Bitmap(Stream streamAppIco)
        {
            Bitmap appIcon32x32Bitmap;
            using (Icon appIcon32x32 = new Icon(streamAppIco, 32, 32))
            {
                appIcon32x32Bitmap = appIcon32x32.ToBitmap();
                appIcon32x32Bitmap.MakeTransparent();
            }
            return appIcon32x32Bitmap;
        }

        public static Bitmap GetTransparentBitmap(Stream streamBitmap)
        {
            Bitmap transparentBitmap = new Bitmap(streamBitmap);
            transparentBitmap.MakeTransparent();
            return transparentBitmap;
        }
    }
}
