﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Reflection;
using System.IO;
using System.Diagnostics;
using Microsoft.Win32;
using DigitaAdvantage.ClickOnce;

namespace DigitaAdvantage.ClickOnce.Forms
{
    internal partial class ErrorPane : MainPane
    {
        private Form parentMainForm;
        private string clickonceLogFilePath;

        public ErrorPane(Form parentForm, string errorTitle, string errorMessage, string logFilePath)
        {
            parentMainForm = parentForm;
            clickonceLogFilePath = logFilePath;

            Assembly asm = DeploymentUtil.GetSystemDeploymentAssembly();
            Stream streamIcoBitmap = asm.GetManifestResourceStream("information.bmp");

            InitializeComponent();

            if (streamIcoBitmap != null)
            {
                pictureBoxIcon.Image = DeploymentUtil.GetTransparentBitmap(streamIcoBitmap); ;
            }

            if (String.IsNullOrEmpty(logFilePath))
            {
                buttonDetail.Enabled = false;
            }

            labelMessage.Text = errorMessage;

            parentMainForm.SuspendLayout();
            parentMainForm.Text = errorTitle;
            parentMainForm.MinimizeBox = false;
            parentMainForm.MaximizeBox = false;
            parentMainForm.ControlBox = true;
            parentMainForm.ResumeLayout(false);
            parentMainForm.PerformLayout();

            if (Form.ActiveForm != parentMainForm)
            {
                parentMainForm.Activate();
            }
        }

        private void ErrorPane_Load(object sender, EventArgs e)
        {
            parentMainForm.ActiveControl = this.buttonOK;
        }

        public override bool OnClosing()
        {
            bool ret = base.OnClosing();
            base.IsError = true;
            return ret;
        }

        private void buttonOK_Click(object sender, EventArgs e)
        {
            base.IsError = true;
        }

        private void buttonDetail_Click(object sender, EventArgs e)
        {
            if (clickonceLogFilePath.Length > 0)
            {
                try
                {
                    Process.Start("notepad.exe", clickonceLogFilePath);
                }
                catch (Win32Exception)
                {
                }
            }
            else
            {
                MessageBox.Show("ログファイルの場所を特定できませんでした。", parentMainForm.Text);
            }
        }
    }
}
