﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Reflection;
using System.IO;
using DigitaAdvantage.ClickOnce;
using System.Diagnostics;

namespace DigitaAdvantage.ClickOnce.Forms
{
    internal partial class UpdatingPane : MainPane
    {
        private Form parentMainForm;

        public UpdatingPane(Form parentForm, string publisherName, string appProductName, string serverName, string supportUrl, string headerTitle, string headerMessage, string iconFilePath)
        {
            parentMainForm = parentForm;

            Assembly asm = DeploymentUtil.GetSystemDeploymentAssembly();
            Stream streamSetup = asm.GetManifestResourceStream("setup.bmp");
            Stream streamAppIco = asm.GetManifestResourceStream("defaultappicon.ico");
            
            InitializeComponent();

            this.SuspendLayout();
            this.labelHeader.Font = new Font(this.labelHeader.Font, this.labelHeader.Font.Style | FontStyle.Bold);
            this.linkLabelAppName.Font = new Font(this.linkLabelAppName.Font, this.linkLabelAppName.Font.Style | FontStyle.Bold);
            this.labelServerName.Font = new Font(this.labelServerName.Font, this.labelServerName.Font.Style | FontStyle.Bold);
            if (streamSetup != null)
            {
                this.pictureBoxTop.Image = new Bitmap(streamSetup);
            }
            if (streamAppIco != null)
            {
                this.pictureBoxAppIcon.Image = DeploymentUtil.GetAppIcon32x32Bitmap(streamAppIco);
            }
            if (String.IsNullOrEmpty(iconFilePath) == false)
            {                
                this.pictureBoxAppIcon.ImageLocation = iconFilePath;
            }
            this.labelHeader.Text = headerTitle;
            this.labelSubHeader.Text = headerMessage;
            this.linkLabelAppName.Text = appProductName;
            this.linkLabelAppName.Links.Clear();
            if (DeploymentUtil.IsValidSupportUrl(supportUrl))
            {
                this.linkLabelAppName.Links.Add(0, appProductName.Length, supportUrl);
            }
            this.labelServerName.Text = serverName;
            this.ResumeLayout(false);
        }

        private void UpdatingPane_Load(object sender, EventArgs e)
        {
            parentMainForm.ActiveControl = this.buttonCancel;
        }

        private void buttonCancel_Click(object sender, EventArgs e)
        {
            base.IsCanceled = true;
        }

        private void linkLabelAppName_LinkClicked(object sender, LinkLabelLinkClickedEventArgs e)
        {
            this.linkLabelAppName.LinkVisited = true;
            string url = e.Link.LinkData.ToString();
            try
            {
                Process.Start(url);
            }
            catch
            {
            }
        }

        public override string ProgressMessageText
        {
            set { this.labelProgressDownloading.Text = value; }
        }

        public override int ProgressPercentage
        {
            set { this.progressBarDownloading.Value = value; }
        }

        public override bool OnClosing()
        {
            bool ret = base.OnClosing();
            base.IsCanceled = true;
            return ret;
        }
    }
}
